declare type In<T> = T extends (...args: infer U) => any ? U : unknown[];
declare type Falsy = false | null | undefined;
/** Strongly typed event source */
interface EventSource<T = any> extends Omit<EventEmitter<T>, 'emit'> {
}
/** Extract an array type of valid event keys */
declare type EventKey<T> = 'emit' | (keyof T & string);
/** Extract the argument/return types of a valid event */
declare type EventArgs<T, K extends string> = K extends 'emit' ? [string, unknown[]] : K extends keyof T & EventKey<T> ? In<T[K]> : unknown[];
/** Extract the listener type for a specific event */
declare type Listener<T = any, K extends string = string> = (...args: EventArgs<T, K>) => boolean | void;
/** An object of event keys and listener values */
declare type ListenerMap<T = any> = Partial<{
    [K in EventKey<T>]: Listener<T, K> | Falsy;
}>;
/** The internal cache of listeners by event key */
declare type ListenerCache<T = any> = {
    [K in EventKey<T>]: Set<Listener<T, K>> | undefined;
} & {
    [key: string]: Set<Listener> | undefined;
};

/** The symbol for storing all listeners */
declare const $listeners: unique symbol;
/** Strongly typed event emitter */
declare class EventEmitter<T = any> {
    protected [$listeners]: {
        [key: string]: Set<Listener> | undefined;
    };
    constructor();
    /** Add a recurring listener */
    on<K extends EventKey<T>, Fn extends Listener<T, K> | Falsy>(key: K, fn: Fn): Fn;
    /** Add many recurring listeners */
    on(map: ListenerMap<T>): this;
    /** Remove all listeners of an event */
    off<K extends EventKey<T>>(key: K): this;
    /** Remove a listener of an event */
    off<K extends EventKey<T>>(key: K, fn: Listener<T, K>): this;
    /** Call the listeners of an event */
    emit<K extends EventKey<T>>(key: K, ...args: EventArgs<T, K>): void;
    /** Check if listeners exist for the given event key. */
    hasListeners(key: EventKey<T>): boolean;
    /** Get the listener cache for the given event key. */
    getListeners<K extends EventKey<T>>(key: K): ReadonlySet<Listener<T, K>>;
    /** Get the listener cache for all event keys. */
    getListeners<K extends EventKey<T>>(): ListenerCache<T>;
    /**
     * Create the `Set<Listener>` for the given event key.
     *
     * Returns the new listener set.
     *
     * Use `return super._addListeners(key)` if you override.
     */
    protected _addListeners(key: string): Set<Listener<any, string>>;
    /**
     * Remove the `Set<Listener>` for the given event key.
     *
     * Use `super._removeListeners(key)` if you override.
     */
    protected _removeListeners(key: string): void;
    /**
     * Add a `Listener` for the given event key.
     *
     * Use `super._addListeners(key, fn)` if you override.
     */
    protected _addListener(key: string, fn: Listener): void;
    /**
     * Remove a `Listener` for the given event key.
     *
     * Returns `true` if the given listener was successfully removed.
     *
     * Use `return super._removeListener(key, fn)` if you override.
     */
    protected _removeListener(key: string, fn: Listener): boolean | undefined;
    /**
     * Invoke listeners of the given event key using the given arguments.
     *
     * Use `super._emit(key, args)` if you override.
     */
    protected _emit(key: string, args: any[]): void;
}

export { EventArgs, EventEmitter, EventKey, EventSource, Listener, ListenerCache, ListenerMap };
